from django.conf.urls.defaults import *

urlpatterns = patterns("",
    url(r"^autocomplete/(?P<app_label>\w+)/(?P<model>\w+)/$", "tagging_ext.views.autocomplete",
            name="tagging_ext_autocomplete"),
    url(r"^autocomplete/$", "tagging_ext.views.autocomplete", name="tagging_ext_autocomplete"),
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Tests client django-tagging-ext view

"""

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client

from tagging_ext.tests.models import BlogPost

class TestClient(TestCase):
    fixtures = ['auth.json']
    urls = 'tagging_ext.tests.urls'

    def setUp(self):
        self.logged_in = self.client.login(username="test", password="test")

        bp = BlogPost.objects.create(
            body="I am a body", tags="fun django")
        bp_chars = BlogPost.objects.create(
            body="I am a post tagged with Japanese and French characters",
            tags="fun django 日本語 français",
        )

    def test_login(self):
        self.assert_(self.logged_in)

    def test_ascii_tags(self):
        autocomplete = reverse('tagging_ext_autocomplete')

        response = self.client.get(autocomplete, {'q': 'dj'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'django')

        response = self.client.get(autocomplete, {'q': 'f'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'fun')

    def test_unicode_tag(self):
        autocomplete = reverse('tagging_ext_autocomplete')

        response = self.client.get(autocomplete, {'q': '日'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, '日本語')

        response = self.client.get(autocomplete, {'q': 'f'})
        self.assertEquals(response.status_code, 200)
        self.assertContains(response, 'français')



"""
Tests basic django-tagging-ext functions

"""

from django.test import TestCase
from django.test.client import Client

class TestSynonyms(TestCase):
    
    def setUp(self):
        pass
 
    def tearDown(self):
        pass

    def test_synonym_creation(self):
        pass


from django import template
from django.conf import settings
from django.template.loader import render_to_string

register = template.Library()


@register.simple_tag
def tag_autocomplete_js(format_string=None):
    """format_string should be ``app_label model counts``
    
    renders 'tagging_ext/tag_autocomplete_js.html"""
    if format_string:
        context_list = format_string.split(' ')
        context = {
            'app_label':context_list[0],'model':context_list[1], 'counts':context_list[2]
        }
    else:
        context = {}
    return render_to_string('tagging_ext/tagging_autocomplete_js.html', context)



@register.inclusion_tag("tagging_ext/tag_list.html")
def show_tags_for(obj):
        
    response = {
        "obj": obj,
        "MEDIA_URL": settings.MEDIA_URL,
    }
    
    # Support for static media if supported
    if hasattr(settings, "STATIC_URL"):
        response['STATIC_URL'] = settings.STATIC_URL
    
    return response

@register.inclusion_tag("tagging_ext/tag_count_list.html")
def show_tag_counts(tag_counts):
    return {"tag_counts": tag_counts}


"""
django-tagging-ext views.py

"""

from sys import stderr

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db import connection
from django.db.models import get_model
from django.http import Http404, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.defaultfilters import slugify

from django.contrib.contenttypes.models import ContentType

from tagging.models import Tag # use these to check for tag content

def get_model_counts(tagged_models, tag):
    """ This does a model count so the side bar looks nice.
    """
    model_counts = []
    for model in tagged_models:
        model['count'] = model['query'](tag).count()
        if model['count']:
            model_counts.append(model)
                 
    return model_counts
    
def check_results(results):
    if not results:
        raise Http404('No available items to display for this tag.')
    
    
def index(request, template_name="tagging_ext/index.html", min_size=0,limit=10):
    """
        min_size: Smallest size count accepted for a tag
        order_by: asc or desc by count
        limit: maximum number of tags to display 
        
        TODO: convert the hand-written query to an ORM call. Right now I know
                this works with Sqlite3 and PostGreSQL.
    """
    query = """
        SELECT tag_item.tag_id as tag_id, COUNT(tag_item.tag_id) as counter 
        FROM tagging_taggeditem as tag_item 
        GROUP BY tag_id
        HAVING COUNT(tag_item.tag_id) > %s
        ORDER BY counter desc
        LIMIT %s
    """

    cursor = connection.cursor()
    cursor.execute(query, [min_size, limit])
    
    results = []
    
    for row in cursor.fetchall():
        try:
            tag=Tag.objects.get(id=row[0])
        except ObjectDoesNotExist:
            continue
            
        if ' ' in tag.name:
            continue
        
        record = dict(
            tag=tag,
            count=row[1]
        )
        results.append(record)    
        
    dictionary = {
        'tags':results
    
    }
    

    return render_to_response(template_name, dictionary,
        context_instance=RequestContext(request))      
    

def tag(request, tag='', template_name="tagging_ext/tag.html", tagged_models=(), default_template=None):
    
    # does the tag actually exist?
    tag = get_object_or_404(Tag, name=tag)
            
    dictionary = { 
        'tag': tag,
        'model_counts': get_model_counts(tagged_models,tag)
    }
    
    return render_to_response(template_name, dictionary,
        context_instance=RequestContext(request))

def tag_by_model(request, tag, model,
                    template_name="tagging_ext/tag_by_model.html",
                    tagged_models=(),
                    default_content_template='tagging_ext/default_template.html'):

    # does the tag actually exist?
    tag = get_object_or_404(Tag, name=tag)
    
    model_counts = get_model_counts(tagged_models, tag)
    
    results = None
    for item in model_counts:
        
        # If the model being displayed is the same as what is being looped          
        if model == slugify(item.get('title', '')):
            
            # Fetch the lambda function which runs the ORM query
            query = item.get('query', None)
            
            if query:
                
                # get the results
                results = query(tag)
                
                # Toss 404 if the results are empty
                check_results(results)                
                
                # And if there is a custom_template, use that.
                # otherwise use the default template
                content_template = item.get('content_template', default_content_template)
                break
    
    # Toss 404 if the results are 0.
    check_results(results)                        
    
    dictionary = { 
        'tag': tag,
        'model': model,
        'model_counts': model_counts,
        'content_template': content_template,
        'results': results
    }
    
    return render_to_response(template_name, dictionary,
        context_instance=RequestContext(request))

def autocomplete(request, app_label=None, model=None):
    """returns ``\\n`` delimited strings in the form <tag>||(#)

    GET params are ``q``, ``limit``, ``counts``, ``q`` is what the user
    has typed, ``limit`` defaults to 10, and ``counts`` can be "model", "all"
    or, if absent, will default to all - ie a site-wide count.
    """

    # get the relevent model if applicable
    if app_label and model:
        try:
            model = ContentType.objects.get(app_label=app_label, model=model)
        except:
            raise Http404
    else:
        model = None
    
    if not request.GET.has_key("q"):
        raise Http404
    else:
        q = request.GET["q"]
    
    # counts can be 'all', 'model' or 'None'
    counts = request.GET.get("counts", "all")
    limit = request.GET.get("limit", 10)
    
    if model:
        tags = Tag.objects.filter(
            items__content_type = model,
            name__istartswith = q
        ).distinct()[:limit]
    else:
        tags = Tag.objects.filter(
            name__istartswith = q
        ).distinct()[:limit]

    if counts == "all":
        l = sorted(list(tags),
            lambda x, y: cmp(y.items.all().count(), x.items.all().count())
        )
        tag_list = "\n".join([ '%s||(%s)' % (tag.name, tag.items.all().count() ) for tag in l if tag])

    elif counts == "model":
        if model:
            l = sorted(list(tags),
                lambda x, y:
                    cmp(y.items.filter(content_type=model).count(), x.items.filter(content_type=model).count())
            )
            tag_list = "\n".join(
                ["%s||(%s)" % (tag.name, tag.items.filter(content_type=model).count()) for tag in l if tag]
            )
        else:
            raise Exception(
                'You asked for a model with GET but did not pass one to the url'
            )
    else:
        tag_list = "\n".join([tag.name for tag in tags if tag])
    return HttpResponse(tag_list)



from django.forms import TextInput
from tagging.fields import TagField
from tagging_ext.widgets import TagAutoCompleteInput

class TagAutocompleteField(TagField):
    def formfield(self, **kwargs):
        defaults = {'widget': TagAutoCompleteInput}
        defaults.update(kwargs)
        return super(TagAutocompleteField, self).formfield(**defaults)



from django import template
from django import forms
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe


class TagAutoCompleteInput(forms.TextInput):
    
    class Media:
        if hasattr(settings, 'STATIC_URL'):
            media_url = settings.STATIC_URL
        else:
            media_url = settings.MEDIA_URL

        css = {
            "all": (media_url + "pinax/css/jquery.autocomplete.css",)
        }
        js = (
            media_url + "pinax/js/jquery-1.3.2.min.js",
            media_url + "pinax/js/jquery.bgiframe.min.js",
            media_url + "pinax/js/jquery.ajaxQueue.js",
            media_url + "pinax/js/jquery.autocomplete.min.js"
        )
    
    def init(self, *args, **kwargs):
        if len(args) == 4:
            self.app_label = args[0]
            self.model = args[1]
            args = args[2:]
            super(TagAutoCompleteInput, self).__init__(*args, **kwargs)
        else:
            super(TagAutoCompleteInput, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        output = super(TagAutoCompleteInput, self).render(name, value, attrs)
        
        return output + mark_safe(u"""
            <script type="text/javascript">
                jQuery("#id_%s").autocomplete("%s", {
                    max: 10,
                    highlight: false,
                    multiple: true,
                    multipleSeparator: " ",
                    scroll: true,
                    scrollHeight: 300,
                    matchContains: true,
                    autoFill: true,
                    formatItem: function(data, i, n, value){
                        return data[0]+" "+data[2];
                    },
                    formatResult: function(data, value){
                        return value;
                    }
                });
            </script>""" % (
                name,
                reverse("tagging_ext_autocomplete", kwargs={
                    "app_label": self.app_label,
                    "model": self.model }
                        if 'app_label' in self.__dict__ else {}
                )
            )
        )



from setuptools import setup, find_packages

version = '0.3.2'

setup(
    name='django-tagging-ext',
    version=version,
    description="Adds in new features to supplement django-tagging",
    long_description=open("README.rst").read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
    ],
    keywords='django,pinax',
    author='Daniel Greenfeld',
    author_email='pydanny@gmail.com',
    url='http://github.com/pydanny/django-tagging-ext',
    license='MIT',
    packages=find_packages(),
)
